"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const knex_1 = __importDefault(require("knex"));
class MariaDBConnector {
    constructor(ip, port, database, user, password) {
        this.database = database;
        this.dbConnection = knex_1.default({
            client: 'mysql',
            connection: {
                database: database,
                host: ip,
                port: port,
                user: user,
                password: password,
                supportBigNumbers: true,
            },
        }).on('query-error', (err) => {
            console.error(err.code, err.name);
        });
    }
    getTablesInformation(ignoredTables) {
        return __awaiter(this, void 0, void 0, function* () {
            const tables = yield this.dbConnection
                .select([
                this.dbConnection.raw('t.TABLE_NAME AS name'),
                this.dbConnection.raw('GROUP_CONCAT(c.REFERENCED_TABLE_NAME SEPARATOR ",") AS referencedTablesString'),
            ])
                .from('information_schema.tables as t')
                .leftJoin('information_schema.key_column_usage as c', function () {
                this.on('c.CONSTRAINT_SCHEMA', '=', 't.TABLE_SCHEMA')
                    .andOn('c.TABLE_NAME', '=', 't.TABLE_NAME');
            })
                .where('t.TABLE_SCHEMA', this.database)
                .andWhere('t.TABLE_TYPE', 'BASE TABLE')
                .whereNotIn('t.TABLE_NAME', ignoredTables)
                .groupBy('t.TABLE_SCHEMA', 't.TABLE_NAME')
                .orderBy(2);
            for (const t in tables) {
                const table = tables[t];
                table.referencedTables = (table.referencedTablesString || '').split(',');
                table.maxLines = yield this.countLines(table);
            }
            return tables;
        });
    }
    getColumnsInformation(table) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.dbConnection.select()
                .from('information_schema.COLUMNS')
                .where({
                'TABLE_SCHEMA': this.database,
                'TABLE_NAME': table.name
            });
        });
    }
    getForeignKeys(table) {
        return __awaiter(this, void 0, void 0, function* () {
            const subQuery = this.dbConnection
                .select([
                'kcu2.table_name',
                'kcu2.column_name',
                'kcu2.constraint_schema',
                this.dbConnection.raw('1 AS unique_index')
            ])
                .from('information_schema.KEY_COLUMN_USAGE AS kcu2')
                .innerJoin('information_schema.TABLE_CONSTRAINTS AS tc', function () {
                this.on('tc.CONSTRAINT_SCHEMA', '=', 'kcu2.CONSTRAINT_SCHEMA')
                    .andOn('tc.TABLE_NAME', '=', 'kcu2.TABLE_NAME')
                    .andOn('tc.CONSTRAINT_NAME', '=', 'kcu2.CONSTRAINT_NAME')
                    .andOnIn('tc.CONSTRAINT_TYPE', ["PRIMARY KEY", "UNIQUE"]);
            })
                .groupBy(['kcu2.TABLE_NAME', 'kcu2.CONSTRAINT_NAME'])
                .having(this.dbConnection.raw('count(kcu2.CONSTRAINT_NAME) < 2'))
                .as('indexes');
            const foreignKeys = yield this.dbConnection.select([
                'kcu.column_name AS column',
                'kcu.referenced_table_name AS foreignTable',
                'kcu.referenced_column_name AS foreignColumn',
                'unique_index AS uniqueIndex'
            ])
                .from('information_schema.key_column_usage as kcu')
                .leftJoin(subQuery, function () {
                this.on('kcu.table_name', 'indexes.table_name')
                    .andOn('kcu.column_name', 'indexes.column_name')
                    .andOn('kcu.constraint_schema', 'indexes.constraint_schema');
            })
                .where('kcu.table_name', table.name)
                .whereNotNull('kcu.referenced_column_name');
            return foreignKeys;
        });
    }
    countLines(table) {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield this.dbConnection(table.name).count())[0]['count(*)'];
        });
    }
    emptyTable(table) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.dbConnection.raw('SET FOREIGN_KEY_CHECKS = 0;');
            yield this.dbConnection.raw(`DELETE FROM ${table.name}`);
            yield this.dbConnection.raw(`ALTER TABLE ${table.name} AUTO_INCREMENT = 1;`);
        });
    }
    getValuesForForeignKeys(table, column, foreignTable, foreignColumn, limit, unique, condition) {
        return __awaiter(this, void 0, void 0, function* () {
            let values = [];
            const query = this.dbConnection(foreignTable)
                .distinct(`${foreignTable}.${foreignColumn}`)
                .orderByRaw('RAND()')
                .limit(limit);
            if (condition) {
                query.andWhere(this.dbConnection.raw(condition));
            }
            if (unique) {
                query.leftJoin(table, function () {
                    this.on(`${table}.${column}`, `${foreignTable}.${foreignColumn}`);
                }).whereNull(`${table}.${column}`);
            }
            values = (yield query).map(result => result[foreignColumn]);
            return values;
        });
    }
    executeRawQuery(query) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.dbConnection.raw(query);
        });
    }
    insert(table, rows) {
        return __awaiter(this, void 0, void 0, function* () {
            const query = yield this.dbConnection(table)
                .insert(rows)
                .toQuery()
                .replace('insert into', 'insert ignore into');
            const insertResult = yield this.dbConnection.raw(query);
            return insertResult[0].affectedRows;
        });
    }
    destroy() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.dbConnection.destroy();
        });
    }
}
exports.MariaDBConnector = MariaDBConnector;
